/* 
 * Copyright (c) 2001 by Matt Welsh and The Regents of the University of 
 * California. All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without written agreement is
 * hereby granted, provided that the above copyright notice and the following
 * two paragraphs appear in all copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR
 * DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT
 * OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF THE UNIVERSITY OF
 * CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 * Author: Matt Welsh <mdw@cs.berkeley.edu>
 * 
 */

package mdw.sandStorm.internal;

import mdw.sandStorm.api.*;
import mdw.sandStorm.api.internal.*;
import mdw.sandStorm.core.*;
import mdw.sandStorm.main.*;
import java.util.*;

/**
 * The ResponseTimeController attempts to keep the response time of
 * a given stage below a given target by adjusting queue thresholds.
 * size of a given ThreadPool.
 * 
 * @author   Matt Welsh
 */
public abstract class ResponseTimeController {

  protected final static int INIT_THRESHOLD = 16;
  protected final static int MIN_THRESHOLD = 1;
  protected final static int MAX_THRESHOLD = 1024;

  protected StageWrapperIF stage;
  protected EnqueuePredicateIF pred;
  protected double targetRT;

  protected ResponseTimeController(ManagerIF mgr, StageWrapperIF stage, boolean setPredicate) throws IllegalArgumentException {
    this.stage = stage;

    SandstormConfig config = mgr.getConfig();
    this.targetRT = config.getDouble("stages."+stage.getStage().getName()+".rtController.targetResponseTime");
    if (this.targetRT == -1) {
      this.targetRT = config.getDouble("global.rtController.targetResponseTime");
      if (this.targetRT == -1) {
	throw new IllegalArgumentException("ResponseTimeController: Must specify targetResponseTime");
      }
    }

    this.pred = stage.getStage().getSink().getEnqueuePredicate();

    if (setPredicate) {
      if (this.pred == null) {
	this.pred = new QueueThresholdPredicate(stage.getStage().getSink(), MAX_THRESHOLD);
	stage.getStage().getSink().setEnqueuePredicate(this.pred);
      } else {
	if (!(pred instanceof QueueThresholdPredicate)) {
	  throw new IllegalArgumentException("ResponseTimeController: Cannot be used with enqueue predicates other than QueueThresholdPredicate");
	}
      }
      ((QueueThresholdPredicate)pred).setThreshold(INIT_THRESHOLD);
    }
  }

  protected ResponseTimeController(ManagerIF mgr, StageWrapperIF stage) {
    this(mgr, stage, true);
  }

  public abstract void adjustThreshold(QueueElementIF fetched[], long serviceTime);

}
