/**
 * STSVerify.
 *
 * @author     Gifford Cheung 
 * @version    $Id: STSVerify.java,v 1.3 2002/04/06 22:52:43 eaton Exp $
 */

package ostore.mac;
import ostore.util.*;
import ostore.security.*;

/**
 * Specifically made for the Station-to-Station Protocol to send
 * public keys back and forth. Used by {@link HMACDHHandler}
 *
 * @author   Gifford Cheung 
 * @version    $Id: STSVerify.java,v 1.3 2002/04/06 22:52:43 eaton Exp $
 */
public class STSVerify implements Digestible {

    private QSDHPublicKey myPu;
    private QSDHPublicKey otherPu;
    private int type;

    private byte[] digest;
    
    /**
     * Constructor.
     */
    public STSVerify ( QSDHPublicKey myPu, 
                      QSDHPublicKey otherPu, 
                      int MACtype ) {

        this.myPu = myPu;
        this.otherPu = otherPu;
        this.type = type;

        // create digest
        digest = encode();
    }

    public byte[] getEncoded() { return digest; }

    /**
     * Note, since STSVerify is only used to transport the <tt> digest
     * </tt>, all this will decode is <tt>digest</tt> and nothing more
     * <br></br>
     * The deserialization constructor.  See the comments at the top of
     * {@link ostore.util.QuickSerializable} for details.
     */
    public STSVerify (byte [] data, int [] offset) throws QSException {
        int dlen = ByteUtils.bytesToInt( data, offset);
        digest = new byte[dlen];    
        offset[0] += ByteUtils.memcpy( digest, 0, data, offset[0], 
                                        dlen);
    }
    /**
     * Note, since STSVerify is only used to transport the <tt> digest
     * </tt>, all this will encode is <tt>digest</tt> and nothing more
     * <br></br>
     */
    public void to_bytes (byte[] data, int[] offset) {
        ByteUtils.intToBytes( digest.length, data, offset);
        offset[0] += ByteUtils.memcpy( data, offset[0], digest, 0,
                                       digest.length);
    }
    public void to_bytes_helper (byte [] data, int [] offset) {
        myPu.to_bytes( data, offset);
        otherPu.to_bytes( data, offset);
        ByteUtils.intToBytes( type, data, offset);
    
    }

    public void add_to_digest(SecureHashBuffer buffer)
    {
        buffer.add(digest.length);
        buffer.add(digest, 0, digest.length);
    }

    /**
     * @return a byte array containing the to_bytes of <tt>this</tt>
     */
    private byte[] encode() {
        int[] offset = {0};
        to_bytes_helper( null, offset);
        byte[] data = new byte[offset[0]];
        offset[0] = 0;
        to_bytes_helper( data, offset);
        return data;
    }

    public int type_code () {
        throw new NoSuchMethodError();
        //return Types.MAC_STSVERIFY;
    }

}
