/**
 * TestStage.
 *
 * @author	Gifford Cheung
 */  

package ostore.mac;
import java.util.*;
import java.math.*;
import java.net.*;
import ostore.util.*;
import ostore.dispatch.*;
import ostore.tapestry.*;
import ostore.network.*;
import ostore.security.*;
import ostore.security.QSRsaPublicKey;
import ostore.security.QSRsaPrivateKey;
import ostore.security.QSDHPublicKey;
import mdw.sandStorm.api.*;
import mdw.sandStorm.internal.SinkProxy;
import mdw.sandStorm.core.ssTimer;

// Security stuff
import java.security.*;
import java.security.spec.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;
import cryptix.jce.provider.*;
import cryptix.jce.provider.random.*;
import cryptix.jce.provider.key.*;
import cryptix.jce.provider.dh.*;
import cryptix.jce.provider.mac.*;


/**
 * TestStage stage for trying out security protocols: DH and HMAC
 */
public class TestStage implements EventHandlerIF, ProfilableIF {

	public static final boolean DEBUG = true;

	public void handleEvent(QueueElementIF item)
		throws EventHandlerException {

        if (item instanceof Blah) {
            Blah msg = (Blah) item;
            System.out.println("MARK.");
            //try {
                _classifier.dispatch_later(msg, 90000000);
            //} catch (SinkException e) {
            //    System.out.println("Oh Brother were art thou.");
            //}
        }
	}

	public void handleEvents(QueueElementIF item[])
		throws EventHandlerException {

		for (int i = 0; i < item.length; i += 1)
			handleEvent(item[i]);

	}

	public void init(ConfigDataIF config) throws Exception {

		System.err.println("TestStage.-------------------------");

        Types.registerAllTypes();
        _classifier = Classifier.getClassifier ( 
                      new NodeId ("chimp.cs.berkeley.edu:0"));

		String stageName = config.getStage().getName();
		try 
		{
		  if(config.getBoolean("Profile"))
		  {
		    config.getManager().getProfiler().add("ostore.mac.TestStage", this);
		    SinkIF sink =
		      config.getManager().getStage(stageName).getSink();
		    _this_sink = new SinkProxy(sink,
					       config.getManager(),
					       config.getStage().getWrapper());
		  }
		  else
		    _this_sink =
		      config.getManager().getStage(stageName).getSink();
		} 
		catch (NoSuchStageException e) 
		{
		  Carp.die (stageName + ".init: could not get own sink.");
		}

		Types.registerAllTypes();

		// where to get my public key... in the near future, in the far
		// future things may be different.
		/** 
	        String keyfilename = config.getString ("keyfile");
        	try {
            		FileInputStream keyfile = new FileInputStream (keyfilename);
            		_qs_public_key  = (QSPublicKey)  QSIO.read (keyfile);
            		_qs_private_key = (QSPrivateKey) QSIO.read (keyfile);
            		keyfile.close ();
        	}
        	catch (Exception e) {
            		BUG ("TestStage.init: caught exception " + e +
                        	"while trying to read keys from disk.");
        	}
		**/

		// subscribe ourselves to certain types of messages
		Filter filter = new Filter();
		if (! filter.requireType (Class.forName
			("ostore.mac.TestStage$Blah")))
		Carp.die ("TestStage: could not require type.");
		_classifier.subscribe( filter, _this_sink);


		try {
			selfTest();
		} catch (Exception e) {
			System.err.println("Exception " + e);
			e.printStackTrace();
		}

        // timing testing
        //try {
            Blah msg = new Blah();
            _classifier.dispatch_later(msg, 5000);
            System.out.println("Mark 1.");
        //} catch ( SinkException e) {
        //    System.out.println("Oh mommy.");
        //}

		System.err.println("TestStage Done.--------------------");

	}


    
	public void selfTest() 
	 throws NoSuchProviderException, 
		NoSuchAlgorithmException, 
		InvalidKeyException,
		InvalidAlgorithmParameterException,
		InvalidParameterSpecException {

		System.err.println("Beginning test.");

	//Initialization stuff

		System.out.println("Adding CryptixJCE");
		Provider cryptixJCE = new cryptix.jce.provider.CryptixCrypto();
		Security.addProvider(cryptixJCE);
		//cryptixJCE.list(System.out);

		System.out.println("Adding CryptixRandom");
		Provider cryptixRandom = new cryptix.jce.provider.CryptixRandom();
		Security.addProvider(cryptixRandom);
		cryptixRandom.list(System.out);
		

		// Initialize SecureRandom (Random number generator)
		boolean available = DevRandom.isAvailable();
		System.err.println("DevRandom.isAvailable = " + available);
		// Note that we are using DevRandom from CryptixRandom
		// which pulls their numbers from /usr/random
		// which, by the way, is butt fast.
		SecureRandom r = SecureRandom.getInstance("DevRandom","CryptixRandom");

	// **DH stuff**
	// DH key generation (Two parts: (1) Generate the DH generation parameters
        //                               (2) Generate the DH pair 
/**
		// (1) Generate DH generation parameters
		System.err.println("Generating a DH generation parameters");
		AlgorithmParameterGenerator dhpg =
			AlgorithmParameterGenerator.getInstance("DH");
		dhpg.init(new javax.crypto.spec.DHGenParameterSpec(4096,4096), r);
		AlgorithmParameters dhp = dhpg.generateParameters();
		AlgorithmParameterSpec dhps = dhp.getParameterSpec(DHParameterSpec.class);
**/
		// (2) Generate DH key pair from those parameters
		System.err.println("---------------------");
		System.err.print("Generating a DH key pair...");
		KeyPairGenerator kpgen = KeyPairGenerator.getInstance("DH","CryptixCrypto");
		// do we know what size we want the DH key to start out as?
		// what size do you want the prime? 3072 bit
		// exponent? 3072 bits as well for security or 160 bit for speed?
		//kpgen.initialize(dhps,r);
		//int keysize = 4096;
		int keysize = 1024;
		kpgen.initialize(keysize, r);

		// generate pair
		KeyPair pair = kpgen.generateKeyPair();
		System.err.println("done.");

		//(2 a) print out
		//currently, CryptixJCE doesn't have getEncoded correctly implemented for 
		//key pairs (both public and private) so we kludge and draw Y out.
		byte[] pu = ((DHPublicKey)pair.getPublic()).getY().toByteArray();
		System.err.println("Public Key: " + 
			" Version UID: " + pair.getPublic().serialVersionUID +
			" Format: " + pair.getPublic().getFormat() +
			" y: " + 
			ByteUtils.print_bytes(pu,0,pu.length) );
		byte[] pr = ((DHPrivateKey)pair.getPrivate()).getX().toByteArray();
		System.err.println("Private Key: " + 
			" Version UID: " + pair.getPrivate().serialVersionUID +
			" Format: " + pair.getPrivate().getFormat() +
			" x: " + 
			ByteUtils.print_bytes(pr,0,pr.length) );
		byte[] aPublic = pu;
/**
		DHPublicKey kppub = pair.getPublic();
		System.err.println("Public Key: " + 
			pair.getPublic().getEncoded().length + " bytes " + 
			ByteUtils.print_bytes(pair.getPublic().getEncoded(), 0, 16));
		System.err.println("Private Key: " + 
			pair.getPrivate().getEncoded().length + " bytes " + 
			ByteUtils.print_bytes(pair.getPrivate().getEncoded(), 0, 16));
**/
                // (3) another key pair
		System.err.println("---------------------");
                System.err.print("Generating a 2nd key pair...");
                KeyPair pair2 = kpgen.generateKeyPair();
                System.err.println("done.");

		// (3 a) print out
		//currently, CryptixJCE doesn't have getEncoded correctly implemented for 
		//key pairs (both public and private) so we kludge and draw Y out.
		pu = ((DHPublicKey)pair2.getPublic()).getY().toByteArray();
		System.err.println("Public Key: " + 
			" Version UID: " + pair2.getPublic().serialVersionUID +
			" Format: " + pair2.getPublic().getFormat() +
			" y: " + 
			ByteUtils.print_bytes(pu,0,pu.length) );
		pr = ((DHPrivateKey)pair2.getPrivate()).getX().toByteArray();
		System.err.println("Private Key: " + 
			" Version UID: " + pair2.getPrivate().serialVersionUID +
			" Format: " + pair2.getPrivate().getFormat() +
			" x: " + 
			ByteUtils.print_bytes(pr,0,pr.length) );
		byte[] bPublic = pu;

		//******************************************
		// (STS Message 1): 	data = public DH Key
		//			type = HMAC-MD5
		//			verify = null
		// no verify work to do just encoding and decoding
		//******************************************

                // generate ZZ from a private key and a public key
                // (4) generate shared secret
		System.err.println("---------------------");
                System.err.print("Generating ZZ...");
                KeyAgreement aKeyAgree = KeyAgreement.getInstance("DH", "CryptixCrypto");
                aKeyAgree.init(pair.getPrivate());
                aKeyAgree.doPhase(pair2.getPublic(), true);
                System.err.println("done.");

		System.err.println("---------------------");
                // (5) generate shared secret...again
                System.err.println("Generating ZZ2...");
                KeyAgreement bKeyAgree = KeyAgreement.getInstance("DH", "CryptixCrypto");
                bKeyAgree.init(pair2.getPrivate());
                bKeyAgree.doPhase(pair.getPublic(), true);

		// (6) compare secret equivalences
		byte[] aZZ = aKeyAgree.generateSecret();
		byte[] bZZ = bKeyAgree.generateSecret();
		System.err.println(ByteUtils.print_bytes(aZZ, 0, aZZ.length));
		System.err.println(" ");
		System.err.println(ByteUtils.print_bytes(bZZ, 0, bZZ.length));
		System.err.print("is ZZ == ZZ2?...");
		if (ByteUtils.equals(aZZ,bZZ)) System.err.println("YES");
		else	System.err.println("NO");

		// (5,6 a) Testing QSDHPublicKey 
		System.err.println("---------------------");
		System.err.println("Testing QSDHPublicKey");
		QSDHPublicKey QSpu = new QSDHPublicKey((DHPublicKey)pair2.getPublic());
		System.err.println("encoding key...");
		int[] o = {0};
		QSpu.to_bytes(null, o);	
		//System.out.println("offset = " + o[0]);
		byte[] encoding = new byte[o[0]];
		o[0] = 0;
		QSpu.to_bytes(encoding, o);

		System.err.println("decoding key...");
		o[0] = 0;
		QSDHPublicKey dQSpu = null;
		try{
			dQSpu = new QSDHPublicKey(encoding, o);
		} catch (QSException e) {Carp.die(e);}
		System.err.println("generating ZZ from it");
		KeyAgreement testKeyAgree = KeyAgreement.getInstance("DH", "CryptixCrypto");
		testKeyAgree.init(pair.getPrivate());
		testKeyAgree.doPhase(dQSpu.key(), true);
		byte[] cZZ = testKeyAgree.generateSecret();
		System.err.print("is ZZ == ZZ2 == ZZ?...");
		if (ByteUtils.equals(aZZ,cZZ)) System.err.println("YES");
		else	System.err.println("NO");
		System.err.println("---------------------");

		
	// **HMAC stuff** 
	// HMAC 128 bit key generation (for use w/ MD5)
		System.err.print("Generating an HMAC secret key...");

		KeyGenerator kgen = KeyGenerator.getInstance ("HMAC","CryptixCrypto");

		kgen.init(128, r);
		SecretKey skey = kgen.generateKey();
		System.err.println("done.");

		System.err.println("Secret Key: " + 
			skey.getEncoded().length + " bytes " + 
			"Format: " + skey.getFormat() + " " +
			ByteUtils.print_bytes(skey.getEncoded(), 0, 16));

	// MAC 128 bit key generation from the DH ZZ seed
		System.err.println("---------------------");
		System.err.println("Initializing HMAC with the DH key.");
		RawSecretKey dhZZ = new RawSecretKey("Diffie-Hellman seed", aZZ);
		Mac hmac = Mac.getInstance ("HMAC-MD5", "CryptixCrypto"); 
		hmac.init(dhZZ);	
		System.err.println("HMAC-MD5 initialized:");
		System.err.println("Mac Length: " + hmac.getMacLength() + " bytes");
		
	
	// MAC digest
		System.err.print("HMACing 'Hi There'...");
		byte[] result = hmac.doFinal("Hi There".getBytes());
		System.err.println("done.");
		System.err.println(ByteUtils.print_bytes(result, 0, result.length));

	// MAC checking.
		System.err.print("checking the Mac for 'Hi There'...");
		hmac.reset();
		hmac.init(dhZZ);
		byte[] resultcheck = hmac.doFinal("Hi There".getBytes());
		System.err.println("equal: " + ByteUtils.equals(result, resultcheck));
		

		//******************************************
		// (STS Message 2):	data = public DH Key
		//			type = HMAC-MD5
		//			verify = MAC(ZZ)(my sign(both public keys 
		//						 and MAC type code);
		//******************************************
	// initialize the signature thing. 

	// signature engine ---> belongs in initProviders() or init Engines();
		System.err.println("---------------------");
		System.err.println("Grabbing RSA KeyPair from quick.priv and quick.pub");
		QSPublicKey QSrpub = null;
		QSPrivateKey QSrpri = null;
		PublicKey rpub = null;
		PrivateKey rpri = null;
		try {
			QSrpub= (QSPublicKey) QSIO.read("quick.pub");
			QSrpri = (QSPrivateKey) QSIO.read("quick.priv");

		 	rpub = QSrpub.key();
		 	rpri = QSrpri.key();

		} catch (Exception e ) {
			Carp.die ("IO Exception reading quick.* for keys: " + e);
		}

		/**
		System.err.println("Generate my own RSA KeyPair");
		KeyPairGenerator rkpgen =
                	KeyPairGenerator.getInstance ("RSA", "Cryptix");
                rkpgen.initialize(keysize, r);
                KeyPair rpair = rkpgen.generateKeyPair();

		QSPublicKey QSrpub =
			new QSRsaPublicKey ((RawRSAPublicKey) (rpair.getPublic ()));
		QSPrivateKey QSrpri =
			new QSRsaPrivateKey ((RawRSAPrivateKey) (rpair.getPrivate()));
		PublicKey rpub = (PublicKey) QSrpub.key();
		PrivateKey rpri = (PrivateKey) QSrpri.key();

		System.out.println("Comparing before and after conversion to QS key: ");
		System.out.println("Public Key  : " +
			ByteUtils.equals(rpair.getPublic().getEncoded(),
					 rpub.getEncoded()));
		System.out.println("Private Key : " +
			ByteUtils.equals(rpair.getPrivate().getEncoded(),
					 rpri.getEncoded()));
		**/



		System.err.println("Setting up signature engine");
		Signature signEngine = null;
		signEngine = Signature.getInstance ("SHA-1/RSA/PKCS#1", "Cryptix");
		
		QSString verifyinfo = new QSString ("NAHAHAHAHAHAH.");
		
		System.err.println("Generating SignedQS encapsulating test string.");
		SignedQS verify = null;
		try {
			verify = new SignedQS ( verifyinfo,
						new QSSha1RsaSignature (),
						rpri,
						r,
						signEngine);
		//	System.out.println("signEngine = " + signEngine);
		}
		catch (Exception e) {
			Carp.die ("Couldn't generate signedQS: " + e);
			
		}
		// Encoding Signed Message (as per self test in SignedQS)
		System.err.println("Encoding SignedQS");
		byte [] encoded_signed_message =
	            new byte [ByteUtils.qs_length (verify)];
       		int [] offset = {0};
        	verify.to_bytes (encoded_signed_message, offset);

		// MACing Encoded Signed Message
		System.err.print("HMACing Signed Message...");
		result = hmac.doFinal(encoded_signed_message);
		System.err.println("done.");
		System.err.println(ByteUtils.print_bytes(result, 0, result.length));

		// Decoding Signed Message
		System.err.println("Decoding SignedQS");
		offset [0] = 0;
		SignedQS decoded = null;
		try {
			decoded = (SignedQS) TypeTable.new_object (
				verify.type_code(),
				encoded_signed_message, offset);
		} catch (Exception e) {
			Carp.die ("Could not decode signed message: " + e);
		}
		
		// Verifying the Signed Message
		System.err.println("Checking the signature.");
		try {
			Assert.assert (
				verify.verify (rpub, signEngine),
				"Signature[source] did not verify.");
			Assert.assert (
				decoded.verify (rpub, signEngine),
				"Signature[dest] did not verify.");
		}
		catch (Exception e) {
			Carp.die ("Could not verify decoded signed message: " + e);
		}

		// then you would check the mac. 
		

		//******************************************
		// (STS Message 3):	data = null 
		//			type = HMAC-MD5
		//			verify = MAC(ZZ)(my sign(both public keys 
		//						 and MAC type code);
		//******************************************
		// same work as above.

			

        // (*) DH key pair generation speed test
		// gibbon.CS.berkeley.EDU
		// recorded numbrs: 4096 bits 5-6 seconds
		// 		     2048 bits <1 second

		// darkseid.CS.berkeley.EDU
		// recorded numbrs: 4096 bits 6-7 seconds
		// 		    1024 bits <1 second
		System.err.println("Diffie Hellman Key Pair Speed Test at " + keysize + " bits");
                System.err.print("Generating a key pair...");
                 pair2 = kpgen.generateKeyPair();
                System.err.println("done.");
                System.err.print("Generating a key pair...");
                 pair2 = kpgen.generateKeyPair();
                System.err.println("done.");
                System.err.print("Generating a key pair...");
                 pair2 = kpgen.generateKeyPair();
                System.err.println("done.");
                System.err.print("Generating a key pair...");
                 pair2 = kpgen.generateKeyPair();
                System.err.println("done.");
                System.err.print("Generating a key pair...");
                 pair2 = kpgen.generateKeyPair();
                System.err.println("done.");
                System.err.print("Generating a key pair...");
                 pair2 = kpgen.generateKeyPair();
                System.err.println("done.");
                System.err.print("Generating a key pair...");
                 pair2 = kpgen.generateKeyPair();
                System.err.println("done.");
                System.err.print("Generating a key pair...");
                 pair2 = kpgen.generateKeyPair();
                System.err.println("done.");

                
                
	}

	public void destroy() throws Exception {
		//TODO
	}

        //  methods for ProfilableIF
        /* Specified by mdw.sandStorm.api.ProfilableIF */
        public int profileSize()
        {
	    return _this_sink.size();
	}
        //  end methods for ProfilableIF

	private SinkIF _this_sink;
    private Classifier _classifier;

    private class Blah implements QueueElementIF {
    }
    

}

				
		
