/**
 * NetworkMessage.
 *
 * @author     Sean C. Rhea
 * @version    $Id: NetworkMessage.java,v 1.7 2001/06/08 23:51:06 srhea Exp $
 */

package ostore.network;
import ostore.util.*;
import mdw.sandStorm.api.*;

public abstract class NetworkMessage 
implements QueueElementIF, QuickSerializable, Cloneable {

    /**
     * If <code>inbound=true</code> then <code>peer</code> is the NodeId
     * of the node where this message came from; otherwise, it's the NodeId
     * of the node where this message should be sent.
     */
    public NodeId peer;
    public boolean inbound;

    public NetworkMessage (NodeId peer, boolean inbound) {
	this.peer = peer;
	this.inbound = inbound;
    }

    public NetworkMessage (byte [] data, int [] offset) {
	// do nothing
    }

    public void to_bytes (byte [] data, int [] offset) {
	// do nothing
    }

    public abstract int type_code ();

    public Object clone () throws CloneNotSupportedException {
	NetworkMessage result = (NetworkMessage) super.clone ();
	result.peer = peer;
	result.inbound = inbound;
	return result;
    }

    public String toString () {
	return "(NetworkMessage " + peer + " "
	    + (inbound ? "inbound" : "outbound") + ")";
    }
}

