/**
 * Test.
 *
 * @author     Sean C. Rhea
 * @version    $Id: Test.java,v 1.4 2001/12/10 23:12:08 geels Exp $
 */

package ostore.network;
import java.io.StreamTokenizer;
import java.io.BufferedReader;
import java.io.FileReader;
import java.net.InetAddress;
import java.util.Random;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.ByteUtils;
import ostore.util.QuickSerializable;
import ostore.util.TypeTable;
import ostore.util.NodeId;
import ostore.dispatch.Filter;
import ostore.dispatch.Signal;
import ostore.dispatch.Classifier;
import mdw.sandStorm.api.ConfigDataIF;
import mdw.sandStorm.api.EventHandlerException;
import mdw.sandStorm.api.EventHandlerIF;
import mdw.sandStorm.api.QueueElementIF;
import mdw.sandStorm.api.SinkIF;
import mdw.sandStorm.api.SinkException;
import mdw.sandStorm.core.ssTimer;
import mdw.sandStorm.api.StagesInitializedSignal;
import mdw.sandStorm.api.NoSuchStageException;
import mdw.sandStorm.api.ProfilableIF;
import mdw.sandStorm.internal.SinkProxy;

public class Test implements EventHandlerIF, ProfilableIF {

    public static class Ping extends NetworkMessage {
	int seq, size;
	public Ping (NodeId peer, int sq, int sz) { 
	    super (peer, false);  
	    seq = sq; 
	    size = sz; 
	}

	public Ping (byte [] data, int [] offset) {
	    super (data, offset);
	    seq = ByteUtils.bytesToInt (data, offset);
	    size = ByteUtils.bytesToInt (data, offset);
	    offset [0] += size;
	}

	public void to_bytes (byte [] data, int [] offset) {
	    ByteUtils.intToBytes (seq, data, offset);
	    ByteUtils.intToBytes (size, data, offset);
	    offset [0] += size;
	}

	public int type_code () { return 1; }

	public Object clone () throws CloneNotSupportedException {
	    Ping result = (Ping) super.clone ();
	    result.seq = seq;
	    result.size = size;
	    return result;
	}

	public String toString () {
	    return "(Ping " + super.toString () + " " + seq + ")";
	}
    }

    public static class Pong extends NetworkMessage {
	int seq, size;
	public Pong (NodeId peer, int sq, int sz) { 
	    super (peer, false);  
	    seq = sq; 
	    size = sz; 
	}

	public Pong (byte [] data, int [] offset) {
	    super (data, offset);
	    seq = ByteUtils.bytesToInt (data, offset);
	    size = ByteUtils.bytesToInt (data, offset);
	    offset [0] += size;
	}

	public void to_bytes (byte [] data, int [] offset) {
	    ByteUtils.intToBytes (seq, data, offset);
	    ByteUtils.intToBytes (size, data, offset);
	    offset [0] += size;
	}

	public int type_code () { return 2; }

	public Object clone () throws CloneNotSupportedException {
	    Pong result = (Pong) super.clone ();
	    result.seq = seq;
	    result.size = size;
	    return result;
	}

	public String toString () {
	    return "(Pong " + super.toString () + " " + seq + ")";
	}
    }

    private static final String class_tag = "Test";

    private static final void BUG (String msg) {
	Carp.die (msg + "  This is a bug in ostore.network.Network.  " +
		"Please let Sean know about it ASAP.");
    }

    private final void dispatch (QueueElementIF item) {
	try {
	    _classifier.dispatch(item);
	} catch (SinkException e) {
	    Carp.out ("Could not dispatch " + item);
	    System.exit (1);
	}
    }

    public void init (ConfigDataIF config) throws Exception {
	
	final String tag = class_tag + ".init";

	String stageName = config.getStage().getName();
	try 
	{
	  if(config.getBoolean("Profile"))
	  {
	    config.getManager().getProfiler().add("ostore.network.Test", this);
	    SinkIF sink =
	      config.getManager().getStage(stageName).getSink();
	    _sink = new SinkProxy(sink,
				  config.getManager(),
				  config.getStage().getWrapper());
	  }
	  else
	    _sink =
	      config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) 
	{
	  Carp.die (tag + ": could not get own sink.");
	}

	TypeTable.register_type ("ostore.network.Test$Ping", 1);
	TypeTable.register_type ("ostore.network.Test$Pong", 2);

	_hostname = config.getString ("hostname");
	_last_seq = config.getInt ("last_seq");
	_starting_port = config.getInt ("starting_port");
	_participants = config.getInt ("participants");
	_my_index = config.getInt ("index");
	_host = InetAddress.getByName (_hostname);

	_peer_seq = new HashMap ();
	_peers_done = new HashSet ();

	int seed = config.getInt ("seed");
	_random = new Random ((long) seed + _my_index);

	_node_id = new MyNodeId (_starting_port + _my_index, _host);
	Carp.out ("I am " + _node_id);

	Debug.turnOn (class_tag);

	Signal.getSignal (_node_id).register (Signal.INIT, null, _sink);

	_classifier = Classifier.getClassifier (_node_id);
	Filter filter = new Filter ();
	if (! filter.requireType (
		    Class.forName ("ostore.network.Test$Ping")))
	    BUG (tag + ": could not require type.");
	if (! filter.requireValue ("inbound", new Boolean (true)))
	    BUG (tag + ": could not require inbound.");
	_classifier.subscribe (filter, _sink);

	filter = new Filter ();
	if (! filter.requireType (
		    Class.forName ("ostore.network.Test$Pong")))
	    BUG (tag + ": could not require type.");
	if (! filter.requireValue ("inbound", new Boolean (true)))
	    BUG (tag + ": could not require inbound.");
	_classifier.subscribe (filter, _sink);

    }

    public void destroy() throws Exception {}

    //  methods for ProfilableIF
    /* Specified by mdw.sandStorm.api.ProfilableIF */
    public int profileSize()
    {
	return _sink.size();
    }
    //  end methods for ProfilableIF

    public void handleEvent(QueueElementIF item) 
    throws EventHandlerException {
	final String tag = class_tag + ".handleEvent";
  	if (item instanceof StagesInitializedSignal) {
	    if (_my_index == _participants) {
		Carp.out ("Starting things up.");
		for (int i = 0; i < _participants; ++i) {
		    NodeId n = new MyNodeId (_starting_port + i, _host);
		    Ping p = new Ping (n, 0, 0);
		    dispatch (p);
		}
	    } 
	}
	else if (item instanceof Ping) {
	    Ping in = (Ping) item;

	    if (in.seq == 0) {
		for (int i = 0; i < _participants; ++i) {
		    if (i == _my_index) continue;
		    NodeId n = new MyNodeId (_starting_port + i, _host);
		    Ping out = new Ping (n, 1, next_size ());
		    _peer_seq.put (n, new Integer (1));
		    dispatch (out);
		}

		for (Iterator j = _peer_seq.keySet ().iterator (); 
			j.hasNext (); ) {
		    NodeId n = (NodeId) j.next ();
		    System.out.print (n.port () + "\t");
		}
		System.out.println ("");
	    }
	    else {
		Pong out = new Pong (in.peer, in.seq, next_size ());
		dispatch (out);
	    }
	}
	else if (item instanceof Pong) {
	    Pong in = (Pong) item;
	    // Carp.out ("Got seq " + in.seq + " from " + in.peer + ".");
	    Integer i = (Integer) _peer_seq.get (in.peer);
	    if (i.intValue () != in.seq) {
		Carp.out ("Bad sequence number.");
		System.exit (1);
	    }

	    _peer_seq.put (in.peer, new Integer (in.seq + 1));

	    for (Iterator j = _peer_seq.keySet ().iterator (); 
		    j.hasNext (); ) {
		NodeId n = (NodeId) j.next ();
		Integer seq = (Integer) _peer_seq.get (n);
		System.out.print (seq + "\t");
	    }
	    System.out.println ("");

	    if (in.seq < _last_seq) {
		Ping out = new Ping (in.peer, in.seq + 1, next_size ());
		dispatch (out);
	    }
	    else {
		_peers_done.add (in.peer);

		if (_peers_done.size () + 1 == _participants)
		    Carp.out ("All done.");
	    }
	}
    }

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for(int i=0; i<items.length; i++)
	    handleEvent(items[i]);
    }

    private int next_size () {
	// TODO: if I were cool, I'd do some neat statistics stuff here.
	// I'm not cool, so I just fudge it.
	return _random.nextInt (10*1024);
    }

    private InetAddress _host;
    private SinkIF _sink;
    private String _hostname;
    private int _starting_port;
    private int _participants;
    private int _my_index;
    private int _last_seq;
    private Map _peer_seq;
    private Set _peers_done;
    private Random _random;
    private NodeId _node_id;
    private Classifier _classifier;
}

