/**
 * PatchworkChangePeriodMsg.java
 *
 * @author Jeremy Stribling
 * @version    $Id: PatchworkChangePeriodMsg.java,v 1.1 2001/12/04 19:51:13 strib Exp $
 */

package ostore.oil.patchwork;
import mdw.sandStorm.api.QueueElementIF;

/**
 * This represents a command to the <code>PatchworkStage</code>
 * indicating that it needs to change the period of one of its
 * messages to a new value (in milliseconds).
 */
public class PatchworkChangePeriodMsg implements QueueElementIF {

    public static final int LATENCY = 0;
    public static final int BANDWIDTH = 1;
    public static final int LOSS = 2;
    public static final int RECALC_MODEL = 3;
   
    public int type;
 
    public int newPeriod;

    public PatchworkChangePeriodMsg( int type, int newPeriod ) {

        if( (type != LATENCY) && (type != BANDWIDTH) &&
            (type != LOSS) && (type != RECALC_MODEL) ) {
          throw new RuntimeException( "Invalid type (" + type +
                                      ") for PatchworkChangePeriodMsg" );
        } else {
          this.type = type;
        }

	if( newPeriod < 0 ) {
          throw new RuntimeException( "New period (" + newPeriod +
                                      ") invalid for PatchworkChangePeriodMsg" 
					+ ", must be >0" );
	} else {
	  this.newPeriod = newPeriod;
	}

    }


    public String toString () {
	return "(PatchworkChangePeriodMsg, type = " + type + 
	", new period = " + newPeriod + ")";
    }
}

