/**
 * QSCipher.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.security;
import ostore.util.QuickSerializable;

/**
 * A class used to map a particular cipher algorithm and provider to a type
 * code, for more efficient encoding.  
 * 
 * @author   Sean C. Rhea
 * @version  $Id: QSCipher.java,v 1.1 2001/10/27 01:00:12 srhea Exp $
 *
 * @see EncryptedQS
 */
public interface QSCipher extends QuickSerializable {

    /**
     * The encryption/decryption algorithm.  Using this and the value of
     * {@link #provider}, a user of this class should be able to produce a
     * cipher to pass to {@link EncryptedQS#decrypt} by calling {@link
     * javax.crypto.Cipher#getInstance(String, String)}.
     *
     * @return the name of the cipher, for instance, "RSA/ECB/PKCS#1"
     */
    public String algorithm ();

    /**
     * The cipher's provider, as used with <code>getInstance</code>.  Using
     * this and the value of {@link #algorithm}, a user of this class
     * should be able to produce a cipher to pass to {@link
     * EncryptedQS#decrypt} by calling {@link
     * javax.crypto.Cipher#getInstance(String, String)}.
     *
     * @return the name of the cipher provider, for instance,
     *         "CryptixCrypto"
     */
    public String provider (); 
}

