/**
 * QSRsaPrivateKey.
 *
 * @author     Sean C. Rhea
 * @version    $Id: QSRsaPrivateKey.java,v 1.4 2002/03/29 23:26:21 eaton Exp $
 */

package ostore.security;
import cryptix.jce.provider.rsa.RSAPrivateCrtKeyCryptix;
import java.math.BigInteger;
import java.security.PrivateKey;
import ostore.util.Assert;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.SecureHashBuffer;
import ostore.util.Types;

/**
 * A subtype of <code>QSPrivateKey</code> for RSA private keys.
 * 
 * @author   Sean C. Rhea
 * @version  $Id: QSRsaPrivateKey.java,v 1.4 2002/03/29 23:26:21 eaton Exp $
 */
public class QSRsaPrivateKey implements QSPrivateKey {

    private static final String class_tag = "QSRsaPrivateKey";

    public QSRsaPrivateKey (PrivateKey key) {
	final String tag = class_tag + ".QSRsaPrivateKey";
	Assert.assert (key != null, tag + ": key must be non-null.");
	Assert.assert (key instanceof RSAPrivateCrtKeyCryptix, 
		tag + ": key must be of type RSAPrivateCrtKeyCryptix, " +
		"not of type " + key.getClass ().getName () + ".");
	_key = (RSAPrivateCrtKeyCryptix) key;
    }

    public QSRsaPrivateKey (byte [] data, int [] offset) throws QSException {
	BigInteger n = ByteUtils.bytesToBigInteger (data, offset);
	BigInteger e = ByteUtils.bytesToBigInteger (data, offset);
	BigInteger d = ByteUtils.bytesToBigInteger (data, offset);
	BigInteger p = ByteUtils.bytesToBigInteger (data, offset);
	BigInteger q = ByteUtils.bytesToBigInteger (data, offset);
	BigInteger pep = ByteUtils.bytesToBigInteger (data, offset);
	BigInteger peq = ByteUtils.bytesToBigInteger (data, offset);
	BigInteger cc = ByteUtils.bytesToBigInteger (data, offset);
	_key = new RSAPrivateCrtKeyCryptix (n, e, d, p, q, pep, peq, cc);
	
	/*
	int length = ByteUtils.bytesToInt (data, offset);
	ByteArrayInputStream is = 
	    new ByteArrayInputStream (data, offset [0], length);
	try {
	    _key = new MyRawRSAPrivateKey (
		    BigInteger.valueOf (0), 
		    BigInteger.valueOf (0));
	    BigInteger d = BI.fromStream(is);
	    BigInteger p = BI.fromStream(is);
	    BigInteger q = BI.fromStream(is);
	    BigInteger u = BI.fromStream(is);
	    _key.mySetRsaParams(d, p, q, u);
	}
	catch (IOException e) {
	    throw new QSException ("Bad encoding.");
	}
	offset [0] += length;
	*/
    }

    public void to_bytes (byte [] data, int [] offset) {
	ByteUtils.bigIntegerToBytes (_key.getModulus (), data, offset);
	ByteUtils.bigIntegerToBytes (_key.getPublicExponent (), data, offset);
	ByteUtils.bigIntegerToBytes (_key.getPrivateExponent (), data, offset);
	ByteUtils.bigIntegerToBytes (_key.getPrimeP (), data, offset);
	ByteUtils.bigIntegerToBytes (_key.getPrimeQ (), data, offset);
	ByteUtils.bigIntegerToBytes (_key.getPrimeExponentP (), data, offset);
	ByteUtils.bigIntegerToBytes (_key.getPrimeExponentQ (), data, offset);
	ByteUtils.bigIntegerToBytes (_key.getCrtCoefficient (), data, offset);

	/*
	byte [] encoded = _key.getEncoded ();
	ByteUtils.intToBytes (encoded.length, data, offset);
	offset [0] += ByteUtils.memcpy (data, offset [0], 
	                                encoded, 0, encoded.length);
	*/
    }

    public int type_code () { return Types.SECURITY_QS_RSA_PRIVATE_KEY; }

    public void add_to_digest(SecureHashBuffer buffer)
    {
        buffer.add (_key.getModulus ());
        buffer.add (_key.getPublicExponent ());
	buffer.add (_key.getPrivateExponent ());
        buffer.add (_key.getPrimeP ());
        buffer.add (_key.getPrimeQ ());
	buffer.add (_key.getPrimeExponentP ());
	buffer.add (_key.getPrimeExponentQ ());
	buffer.add (_key.getCrtCoefficient ());
    }

    public PrivateKey key () {
	return _key;
    }

    protected RSAPrivateCrtKeyCryptix _key;

}

