/**
 * GenerateThresholdKeyShares.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.security.thresh;
import java.security.Security;
import java.security.Provider;
import java.io.IOException;
import ostore.util.QSIO;

/**
 * Generates a set of key shares from the command line.
 *
 * @author  Sean C. Rhea
 * @version $Id: GenerateThresholdKeyShares.java,v 1.2 2001/10/26 23:51:29 srhea Exp $
 */
public class GenerateThresholdKeyShares {

    static {
	Security.addProvider (new cryptix.jce.provider.CryptixCrypto ());
	Security.addProvider (new ThresholdProvider ());
    }

    public static void main (String [] argv) {

	if (argv.length < 4) {
	    System.err.println ("usage: genkey <prefix> <size> " + 
		    "<total shares> <needed to sign>");
	    System.exit (1);
	}

	int which = 0;
	String prefix = argv[which++];
	int keysize = Integer.parseInt (argv[which++]);
	int l = Integer.parseInt (argv[which++]);
	int k = Integer.parseInt (argv[which++]);

	Dealer d = new Dealer("CryptixCrypto",keysize);
	d.generateKeys(k,l);
	ThresholdPublicKey public_key = d.getPublicKey ();
	KeyShare[] keys = d.getShares();

	try {
	    QSIO.write (public_key, prefix + ".pub");
	}
	catch (IOException e) {
	    System.err.println ("Could not write " + prefix + ".pub");
	    System.exit (1);
	}

	for (int i = 0; i < keys.length; ++i) {
	    final String name = prefix + "-" + i + ".priv";
	    try {
		QSIO.write (keys [i], name);
	    }
	    catch (IOException e) {
		System.err.println ("Could not write " + name + ".");
		System.exit (1);
	    }
	}
    }
}
