/**
 * ThresholdSignedQS.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.security.thresh;
import ostore.security.SignedQS;
import ostore.security.QSSignature;
import ostore.util.QuickSerializable;
import ostore.util.ByteUtils;
import ostore.util.TypeTable;
import ostore.util.QSException;

/**
 * A SignedQS for threshold signatures.
 *
 * @author  Sean C. Rhea
 * @version $Id: ThresholdSignedQS.java,v 1.4 2002/04/04 01:46:20 geels Exp $
 */
public class ThresholdSignedQS extends SignedQS {

    /**
     * Deserialize the same way as a SignedQS.
     */
    public ThresholdSignedQS (byte [] data, int [] offset) throws QSException {
	super( data, offset );
    }
    
    /**
     * The default constructor.
     *
     * <p>TODO: this interface practically encourages bugs.
     */
    public ThresholdSignedQS (QuickSerializable user_data, byte [] signature) {

	_user_data = user_data;
	_qs_signature = new ThresholdSignature ();
	_qs_signature_type = TypeTable.type_code (_qs_signature);
	_bytes = data_to_sign (user_data, _qs_signature_type);
	_signature = signature;
    }

    public static SigShare create_sig_share (KeyShare keyshare, 
	    QuickSerializable user_data, long qs_signature_type) {
	return keyshare.sign (data_to_sign (user_data, qs_signature_type));
    }

    public static byte [] data_to_sign (QuickSerializable user_data,
	    long qs_signature_type) {
	byte [] result = null;
	while (true) {
	    int [] offset = {0};
	    ByteUtils.longToBytes (TypeTable.type_code (user_data), 
		    result, offset);
	    user_data.to_bytes (result, offset);
	    ByteUtils.longToBytes (qs_signature_type, result, offset);
	    if (result == null) 
		result = new byte [offset [0]];
	    else
		break;
	}
	return result;
    }
}

