/**
 * RouteTMessage
 *
 * @author     Ben Y. Zhao
 * @version    $Id: RouteTMessage.java,v 1.4 2001/08/16 23:21:18 srhea Exp $
 */

package ostore.tapestry;
import ostore.util.*;
import mdw.sandStorm.api.*;

/*********************
 * Event object generated when a stage wants to send a single message of
 * arbitrary size via Tapestry directly to a known server with a given GUID.
 * Note: can asynchronously generate NodeNotFoundTErrorEvent if exact
 * match requested, and not found.
 * Exact match can be turned off in order to support searching out of 
 * dissemination candidates for newly generated fragments in the OceanStore
 * archival system.
 */
public abstract class RouteTMessage extends TMessageEvent {

    /**
     * Whether the destination must exactly match NodeGuid
     */
    public boolean exact_match;

    public RouteTMessage(byte [] data, int [] offset) {
	super (data, offset);
    }

    public RouteTMessage(SecureHash nGuid, boolean eMatch) {
	super(nGuid,
		false, /* false, not inbound */
		true, /* using MAC by default */
		false); /* verified N/A */
	exact_match = eMatch;
    }

    public RouteTMessage(SecureHash nGuid, boolean eMatch,
	    boolean useMAC) {
	super(nGuid, false, useMAC, false);
	exact_match = eMatch;
    }

    public String toString() {
	return "(RouteTMessage "+super.toString()+")";
    }

    public Object clone() throws CloneNotSupportedException {
	RouteTMessage result = (RouteTMessage) super.clone();
	result.exact_match = exact_match;
	return result;
    }
}

