/**
 * TapestryQueryResultMatch.
 *
 * @author     Sean C. Rhea
 * @version    $Id: TapestryQueryResultMatch.java,v 1.1 2001/09/11 02:40:55 srhea Exp $
 */
package ostore.tapestry.api;
import ostore.util.Assert;
import ostore.util.SecureHash;

/**
 * A result indicating that the given Tapestry tag (and possibly others)
 * satisfy the given query.
 *
 * @see TapestryQuery
 */
public class TapestryQueryResultMatch implements TapestryQueryResult {

    private static final String class_tag = "TapestryQueryResultMatch";

    /**
     * Construct according to the given fields.
     */
    public TapestryQueryResultMatch (SecureHash matches [], boolean stop) {
	final String tag = class_tag + ".TapestryQueryResultMatch";
	Assert.assert (matches.length > 0, 
		tag + ": matches.length must be > 0");
	for (int i = 0; i < matches.length; ++i) 
	    Assert.assert (matches [i] != null, 
	    	tag + ": matches [" + i + "] == null");
	_matches = matches;
	_stop = stop;
    }
    
    /**
     * A list of backpointers to follow.
     */
    public SecureHash [] matches () { return _matches; }

    /**
     * Whether the location operation should stop or continue up the tree.
     */
    public boolean stop () { return _stop; }

    private SecureHash [] _matches;
    private boolean _stop;
}

