/**
 * TapestryRouteMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: TapestryRouteMsg.java,v 1.3 2002/04/06 22:52:43 eaton Exp $
 */
package ostore.tapestry.api;
import ostore.util.SecureHash;
import ostore.util.Digestible;
import mdw.sandStorm.api.QueueElementIF;

/**
 * Messages sent from one OceanStore machine to any other.  To send a
 * message from one OceanStore machine to any other, one simply inherits
 * from this class, filling in the proper serialization code for the
 * inherited class's data.  (The Tapestry stage will take care of the
 * fields in this class itself.)  
 *
 * <p> Implementation-wise, this is simply a route-to-root operation.
 */
public abstract class TapestryRouteMsg 
implements QueueElementIF, Digestible {

    /** 
     * If <code>inbound</code>, the sender; if <code>! inbound</code> the
     * intended recipient.
     */
    public SecureHash peer;

    /**
     * Whether this message is being received (<code>true</code>) or sent
     * (<code>false</code>).
     */
    public boolean inbound;

    /**
     * Construct with the given destination.  Use for new outbound
     * messages.
     */
    protected TapestryRouteMsg (SecureHash peer) {
	this.peer = peer;
	this.inbound = false;
    }

    /**
     * Empty constructor.  For use by derived types when reconstructing
     * themselves from a byte array.
     */
    protected TapestryRouteMsg () {}
}

