/**
 * DynamicDMAckMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicDMAckMsg.java,v 1.1 2002/04/05 06:18:36 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;

/**
 * Message used by a new node to do directed multicast to all nodes 
 * sharing a common suffix.
 */
public class DynamicDMAckMsg extends NetworkMessage {

  public SecureHash src;  //same src as the DMCastMsg (root of mc tree)
  public SecureHash newGuid;  //guid of new node
  public NodeId newAddr;      //nodeid of new node
  public int step;
  public int ttl;

  public DynamicDMAckMsg (NodeId peer, SecureHash src, 
			  SecureHash newGuid, NodeId newAddr,
			  int step, int ttl) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.newGuid = newGuid;
    this.newAddr = newAddr;
    this.step = step;
    this.ttl = ttl;
  }

  public DynamicDMAckMsg (byte [] data, int [] offset) throws QSException {
    super (data, offset);
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
    
    type = ByteUtils.bytesToInt(data, offset);
    newGuid = (SecureHash) TypeTable.new_object(type, data, offset);
    newAddr = new NodeId(data, offset);

    step = ByteUtils.bytesToInt(data, offset);
    ttl = ByteUtils.bytesToInt(data, offset);
  }

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);

    ByteUtils.intToBytes(newGuid.type_code(), data, offset);
    newGuid.to_bytes(data, offset);
    newAddr.to_bytes(data, offset);

    ByteUtils.intToBytes(step, data, offset);
    ByteUtils.intToBytes(ttl, data, offset);
  }

  public int type_code() {
    return 0;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicDMAckMsg result = (DynamicDMAckMsg) super.clone();
    result.src = src;
    result.newGuid = newGuid;
    result.newAddr = newAddr;
    result.step = step;
    result.ttl = ttl;
    return result;
  }

  public String toString() {
    return "(DynamicDMAckMsg: mcastSrc="+src+", step="+step+
      ", ttl="+ttl+")";
  }

}
