/**
 * DynamicDMCastMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicDMCastMsg.java,v 1.6 2002/04/05 06:18:36 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;
// import ostore.mac.MACedQS;

/**
 * Message used by a new node to do directed multicast to all nodes 
 * sharing a common suffix.
 */
public class DynamicDMCastMsg extends NetworkMessage {

  public SecureHash src; //root of mcast tree
  public NodeId srcAddr;
  public SecureHash newGuid;  //guid of new node
  public NodeId newAddr;      //nodeid of new node
  public int step;
  public int ttl;

  public DynamicDMCastMsg (NodeId peer, SecureHash src, NodeId srcAddr,
			   SecureHash newGuid, NodeId newAddr, 
			   int step, int ttl) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.srcAddr = srcAddr;
    this.newGuid = newGuid;
    this.newAddr = newAddr;
    this.step = step;
    this.ttl = ttl;
  }

  public DynamicDMCastMsg (byte [] data, int [] offset) throws QSException {
    super (data, offset);
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
    srcAddr = new NodeId(data, offset);

    type = ByteUtils.bytesToInt(data, offset);
    newGuid = (SecureHash) TypeTable.new_object(type, data, offset);
    newAddr = new NodeId(data, offset);

    step = ByteUtils.bytesToInt(data, offset);
    ttl = ByteUtils.bytesToInt(data, offset);
  }

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
    srcAddr.to_bytes(data, offset);

    ByteUtils.intToBytes(newGuid.type_code(), data, offset);
    newGuid.to_bytes(data, offset);
    newAddr.to_bytes(data, offset);

    ByteUtils.intToBytes(step, data, offset);
    ByteUtils.intToBytes(ttl, data, offset);
  }

  public int type_code() {
    return Types.TAPESTRY_DYNAMIC_MCAST_MSG;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicDMCastMsg result = (DynamicDMCastMsg) super.clone();
    result.src = src;
    result.srcAddr = srcAddr;
    result.newGuid = newGuid;
    result.newAddr = newAddr;
    result.step = step;
    result.ttl = ttl;
    return result;
  }

  public String toString() {
    return "(DynamicDMCastMsg: newGuid="+newGuid+", root="+src+
      ", goingto="+peer+", step="+step+ ", ttl="+ttl+")";
  }

}
