/**
 * DynamicObjReqMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicObjReqMsg.java,v 1.7 2002/03/27 00:08:07 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;

public class DynamicObjReqMsg extends NetworkMessage {

  public SecureHash src;  //source of message
  public SecureHash dest; //desired new ID
  public int msgtype;     //is this request(0) or confirmation(1)

  public static final int REQUEST = 0;
  public static final int CONFIRM = 1;

  public DynamicObjReqMsg (NodeId peer, SecureHash src, SecureHash dest,
			   int msgtype) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.dest = dest;
    this.msgtype = msgtype;
  }

  public DynamicObjReqMsg (byte [] data, int [] offset) throws QSException {
    super (data, offset);
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
    type = ByteUtils.bytesToInt(data, offset);
    dest = (SecureHash) TypeTable.new_object(type, data, offset);
    msgtype = ByteUtils.bytesToInt(data, offset);
  }

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
    ByteUtils.intToBytes(dest.type_code(), data, offset);
    dest.to_bytes(data, offset);
    ByteUtils.intToBytes(msgtype, data, offset);
  }

  public int type_code() {
    return Types.TAPESTRY_DYNAMIC_OBJREQ_MSG;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicObjReqMsg result = (DynamicObjReqMsg) super.clone();
    result.src = src;
    result.dest = dest;
    result.msgtype = msgtype;
    return result;
  }

  public String toString() {
    if (msgtype == REQUEST) 
      return "(DynamicObjReqMsg: Request-->"+dest+")";
    else
      return "(DynamicObjReqMsg: Confirm-->"+dest+")";
  }

}
