/**
 * DynamicRequestMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicRequestMsg.java,v 1.5 2002/04/05 06:18:36 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;

public class DynamicRequestMsg extends NetworkMessage {

  public SecureHash src;
  public int hop;

  public DynamicRequestMsg (NodeId peer, SecureHash src, int hop) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.hop = hop;
  }

  public DynamicRequestMsg (byte [] data, int [] offset) throws QSException {
    super(data, offset);
    System.out.println("requestMsg hash="+(new ostore.util.SHA1Hash(data))+")");
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
    hop = ByteUtils.bytesToInt(data, offset);
    if (data != null) 
      System.out.println("requestMsg hash="+
			 (new ostore.util.SHA1Hash(data)));
  }

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
    ByteUtils.intToBytes(hop, data, offset);
    if (data != null) 
      System.out.println("requestMsg hash="+
			 (new ostore.util.SHA1Hash(data)));
  }

  public int type_code() {
    return Types.TAPESTRY_DYNAMIC_REQ_MSG;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicRequestMsg result = (DynamicRequestMsg) super.clone();
    result.src = src;
    result.hop = hop;
    return result;
  }

  public String toString() {
    return "(DynamicRequestMsg: hop="+hop+", src="+src+", peer="+peer+")";
  }

}
