/**
 * DynamicSurrMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicSurrMsg.java,v 1.4 2001/12/06 22:48:29 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;

public class DynamicSurrMsg extends NetworkMessage {

  public SecureHash src;
  public NodeId nextAddr;
  public int hop;

  public DynamicSurrMsg (NodeId peer, SecureHash src, 
			 NodeId nextAddr, int hop) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.hop = hop;
    this.nextAddr = nextAddr;
  }

  public DynamicSurrMsg (byte [] data, int [] offset) throws QSException {
    super (data, offset);
    int type = ByteUtils.bytesToInt(data, offset);
    src = (SecureHash) TypeTable.new_object(type, data, offset);
    nextAddr = new NodeId(data, offset);
    hop = ByteUtils.bytesToInt(data, offset);
  }

  public void to_bytes(byte [] data, int [] offset) {
    super.to_bytes(data, offset);
    ByteUtils.intToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
    nextAddr.to_bytes(data, offset);
    ByteUtils.intToBytes(hop, data, offset);
  }

  public int type_code() {
    return Types.TAPESTRY_DYNAMIC_SURR_MSG;
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicSurrMsg result = (DynamicSurrMsg) super.clone();
    result.src = src;
    result.nextAddr = nextAddr;
    result.hop = hop;
    return result;
  }

  public String toString() {
    return "(DynamicSurrMsg: hop="+hop+")";
  }

}
