/**
 * PublishInfo.java 
 *
 * @author Ben Y. Zhao
 * @version $Id: PublishInfo.java,v 1.6 2002/03/28 22:57:46 srhea Exp $
 */

package ostore.tapestry.impl;
import java.io.*;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.TypeTable;
import ostore.util.Assert;
import ostore.util.Types;
import ostore.util.QSException;
import ostore.tapestry.api.TapestryTag;
import ostore.util.NodeId;

public class PublishInfo implements Comparable, QuickSerializable {
  public SecureHash src;
  public long distance;
  public TapestryTag tag;
  public NodeId lasthop;

  //  public static final boolean DEBUG = ostore.util.DebugFlags.PUBLISHINFO;

  public PublishInfo (SecureHash src, TapestryTag tag, NodeId lasthop) {
    this.src = src;
    this.distance = Long.MAX_VALUE;
    this.tag = tag;
    this.lasthop = lasthop;
  }
  
  public PublishInfo (SecureHash src, long distance, TapestryTag tag,
		      NodeId lasthop) {
    this.src = src;
    this.distance = distance;
    this.tag = tag;
    this.lasthop = lasthop;
  }
  
  public PublishInfo (byte [] data, int [] offset) throws QSException {
    long hashType = ByteUtils.bytesToLong(data, offset);
    this.src = (SecureHash) TypeTable.new_object(hashType, data, offset);
    this.distance = ByteUtils.bytesToLong(data, offset);
    long type = ByteUtils.bytesToLong(data, offset);
    if (type != 0) 
      tag = (TapestryTag) TypeTable.new_object(type, data, offset);
    else tag = null;
    this.lasthop = new NodeId(data, offset);
  }

  public void to_bytes (byte [] data, int [] offset) {
    ByteUtils.longToBytes(src.type_code(), data, offset);
    src.to_bytes(data, offset);
    ByteUtils.longToBytes(distance, data, offset);
    if (tag == null) ByteUtils.longToBytes(0, data, offset);
    else {
      ByteUtils.longToBytes(TypeTable.type_code(tag), data, offset);
      tag.to_bytes(data, offset);
    }
    lasthop.to_bytes(data, offset);
  }

  public int type_code() {
    return Types.TAPESTRY_PUBLISH_INFO;
  }

  public int compareTo (Object o) {
    Assert.assert (false, "don't use this function; it does not love you.");
    PublishInfo other = (PublishInfo) o;
    long diff = distance - other.distance;
    if (diff != 0)
	return (int) diff;
    if (equals (other))
	return 0;
    return hashCode () - other.hashCode ();
  }

  /**
   * Don't look at distance.
   */
  public boolean equals (Object other) {
    if (! (other instanceof PublishInfo))
    	return false;
    PublishInfo rhs = (PublishInfo) other;
    if (! src.equals (rhs.src))
	return false;

    if (tag == null)
	return rhs.tag == null;
    else if (rhs.tag == null)
	return false;
    else
	return tag.equals (rhs.tag);
  }

  /**
   * Don't look at distance.
   */
   public int hashCode () {
   	int result = src.hashCode ();
	if (tag != null) {
	    result ^= tag.hashCode ();
	}
	return result;
   }

    /**
     * Returns a human-readable representation of this
     * <code>PublishInfo</code>. */
    public String toString()
    {
      String str = new String();

      str += "<PublishInfo";
      str += " tag==" + tag;
      str += " distance==" + distance;
      str += " src==" + src;
      str += " lasthop==" + lasthop;
      str += ">";
      
      return str;
    }
}

