/**
 * PublishMsg.
 *
 * Internal messages for doing either publishing or unpublishing an object 
 * in the Tapestry network.  Whether it's publish or unpublish is indicated
 * by a boolean flag. Publish=true, unpublish=false. This internal message 
 * is routed between Tapestry nodes, where each overlay node examines the
 * message and does the correct thing w.r.t. their local object location
 * pointer map.
 *
 * @author     Ben Y. Zhao
 * @version    $Id: PublishMsg.java,v 1.5 2002/03/28 22:57:46 srhea Exp $
 */

package ostore.tapestry.impl;
import ostore.tapestry.api.TapestryTag;
import ostore.network.NetworkMessage;
import ostore.util.Types;
import ostore.util.NodeId;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.ByteUtils;
import ostore.util.TypeTable;
import ostore.util.QSException;

public class PublishMsg extends NetworkMessage {

    public SecureHash src;
    public SecureHash guid;
    public TapestryTag tag;
    public int hop;
    public boolean publish; /*true if publishing, false if unpublishing*/

    public PublishMsg (NodeId peer, SecureHash src, SecureHash guid,
	    TapestryTag tag, int hop, boolean publish) {
	super (peer, false /* outbound */);
	this.src = src;
	this.guid = guid;
	this.tag = tag;
	this.hop = hop;
	this.publish = publish;
    }

    public PublishMsg (byte [] data, int [] offset) throws QSException {
	super (data, offset);
	long type = ByteUtils.bytesToLong (data, offset);
	src = (SecureHash) TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToLong (data, offset);
	guid = (SecureHash) TypeTable.new_object (type, data, offset);
	type = ByteUtils.bytesToLong (data, offset);
	if (type != 0)
	    tag = (TapestryTag) TypeTable.new_object (type, data, offset);
	hop = ByteUtils.bytesToInt(data, offset);
	publish = ByteUtils.bytesToBool(data, offset);
    }

    public void to_bytes (byte [] data, int [] offset) {
	super.to_bytes (data, offset);
	ByteUtils.longToBytes (TypeTable.type_code (src), data, offset);
	src.to_bytes (data, offset);
	ByteUtils.longToBytes (TypeTable.type_code (guid), data, offset);
	guid.to_bytes (data, offset);
	if (tag == null) 
	    ByteUtils.longToBytes (0, data, offset);
	else {
	    ByteUtils.longToBytes (TypeTable.type_code (tag), data, offset);
	    tag.to_bytes (data, offset);
	}
	ByteUtils.intToBytes(hop, data, offset);
	ByteUtils.boolToBytes(publish, data, offset);
    }

    public int type_code () {
	return Types.TAPESTRY_PUBLISH_MSG;
    }

    public Object clone () throws CloneNotSupportedException {
	PublishMsg result = (PublishMsg) super.clone ();
	result.src = src;
	result.guid = guid;
	result.tag = tag;
	result.hop = hop;
	result.publish = publish;
	return result;
    }

    public String toString () {
      if (publish)
      	return "(Publish" +
	    " hop=" + hop +
	    " guid=" + guid +
	    " tag=" + tag +
	    " src=" + src +
	    " " + super.toString () + 
	    ")";
      else 
	return "(Unpublish" +
	    " hop=" + hop +
	    " guid=" + guid +
	    " tag=" + tag +
	    " src=" + src +
	    " " + super.toString () + 
	    ")";
    }
}

