/*
 *  StaticTClient.java
 *
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import mdw.sandStorm.api.ConfigDataIF;
import mdw.sandStorm.api.EventHandlerException;
import mdw.sandStorm.api.EventHandlerIF;
import mdw.sandStorm.api.QueueElementIF;
import mdw.sandStorm.api.SinkIF;
import mdw.sandStorm.api.SinkException;
import mdw.sandStorm.api.StagesInitializedSignal;
import mdw.sandStorm.api.ProfilableIF;
import mdw.sandStorm.api.NoSuchStageException;
import mdw.sandStorm.internal.SinkProxy;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.dispatch.Classifier;
import ostore.util.Carp;
import ostore.dispatch.Filter;
import ostore.util.SecureHash;
import ostore.util.NodeId;
import ostore.util.Assert;
import ostore.util.Clock;
import ostore.tapestry.api.TapestryInstance;
import ostore.oil.patchwork.PatchworkProbeMsg;

/**
 * A Tapestry client for statically configured networks.
 *
 * @author     Sean C. Rhea
 * @version    $Id: StaticTClient.java,v 1.38 2002/03/28 20:19:23 eaton Exp $
 */
public class StaticTClient implements EventHandlerIF, ProfilableIF {

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for (int i=0; i < items.length; i++)
	    handleEvent (items[i]);
    }

    public void destroy() throws Exception {
	// TODO
    }

    //  methods for ProfilableIF
    public int profileSize()
    {
	return _sink.size();
    }
    //  end methods for ProfilableIF

    private String class_tag = "StaticTClient";
    private static final boolean DEBUG = DebugFlags.TAPESTRY_STATIC_TCLIENT;

    private void dispatch (QueueElementIF item) {
	final String tag = class_tag + ".dispatch";
	if (DEBUG) Debug.printtagln (tag, "dispatching " +
		item.getClass () + " event: " + item + ".");
	try {
	    Classifier.getClassifier(_self_node_id).dispatch(item);
	} catch (SinkException e) {
	    BUG (class_tag + ".dispatch: could not dispatch " + item);
	}
    }

    private void dispatch (QueueElementIF item, long delay) {
	final String tag = class_tag + ".dispatch";
	if (DEBUG) Debug.printtagln (tag, "dispatching " +
		item.getClass () + " event: " + item + " " +
				     delay + "ms later.");

	Classifier.getClassifier(_self_node_id).dispatch_later(item, delay);
    }

    private static final void BUG (String msg) {
	Carp.die (msg + "  This is a bug in " +
		"ostore.tapestry.impl.StaticTClient.  " +
		"Please let Sean know about it ASAP.");
    }
    
    public void init (ConfigDataIF config) throws Exception {
	_self_node_id = new NodeId (config.getString ("node_id"));
	if (DEBUG) Debug.turnOn (class_tag);
	class_tag = class_tag + ".[" + _self_node_id.port () + "]";
	final String tag = class_tag + ".init";
	_timing_map = null;
	_out_times = null;
	_fed_node_id = new NodeId (config.getString ("federation"));
	String _dynamic_route = config.getString("dynamic_route");
	if (_dynamic_route != null && _dynamic_route.equals("dynamic")) {
	  return;
	}

        _ping_delay = config.getInt("PingDelay");
        if (_ping_delay == -1)
        {
            _ping_delay = 5000;
        }

	_simulator_running = config.getBoolean("simulator_running");

	String stageName = config.getStage().getName();
	try 
	{
	  if(config.getBoolean("Profile"))
	  {
	      config.getManager().getProfiler().add("ostore.tapestry.impl.StaticTClient", this);
	      SinkIF sink =
		  config.getManager().getStage(stageName).getSink();
	      _sink = new SinkProxy(sink,
			      config.getManager(),
			      config.getStage().getWrapper());
	  }
	  else
	      _sink =
		  config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) 
	{
	    Carp.die (tag + ": could not get own sink " + e + ".");
	}

	String [] event_types = {
	    "mdw.sandStorm.api.StagesInitializedSignal"
	};

	for (int i = 0; i < event_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (event_types [i])))
		BUG (tag + ": could not require type " + event_types [i]);
	    Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}

	String [] inb_msg_types = {
	    "ostore.tapestry.impl.StaticAllNodesMsg",
	    "ostore.tapestry.impl.StaticBeginMsg",
	    "ostore.tapestry.impl.StaticPingMsg",
	    "ostore.tapestry.impl.StaticPongMsg",
	};

	for (int i = 0; i < inb_msg_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (inb_msg_types [i])))
		BUG (tag + ": could not require type " + inb_msg_types [i]);
	    if (! filter.requireValue ("inbound", new Boolean (true)))
		BUG (tag + ": could not require inbound = true for " 
			+ inb_msg_types [i]);
	    Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}

        // require probe requests
        try {
          Filter filter = new Filter();
          Class probeClass = Class.forName( 
				"ostore.oil.patchwork.PatchworkProbeMsg" );
          if( DEBUG ) Debug.printtagln( tag, "Requiring probe class" );
        
          if( !filter.requireType( probeClass ) ||
              !filter.requireValue ( "inbound", new Boolean(true) ) ||
              !filter.requireValue ( "response", new Boolean(false) ) ||
	      !Classifier.getClassifier(_self_node_id).subscribe 
							(filter, _sink) ) {
              
              Carp.die( "Could not require probe class " );
     
          }
        } catch ( ClassNotFoundException cnfe ) {
            Carp.die( cnfe );
	}

    }

    protected void handle_stages_init_signal () {
	final String tag = class_tag + ".handle_stages_init_signal";

	if (DEBUG) Debug.printtagln (tag,
		"processing StagesInitializedSignal" );

	_self_guid = TapestryInstance.instance (_self_node_id).self_guid ();
	_self_digits = RoutingTable.digits (_self_guid);

        //  PRE: Take a little nap here so that the network stage on
        //  the federation has time to finish initializing.
        if (! _simulator_running)
        {
            try
            {
                Thread.sleep(5000);
            }
            catch (InterruptedException e)
            {
                //  nothing
            }
        }

	StaticAliveMsg alive_msg = new StaticAliveMsg (
		_fed_node_id, _self_guid);
	dispatch (alive_msg);
    }

    protected void handle_static_all_nodes_msg (StaticAllNodesMsg msg) {
	final String tag = class_tag + ".handle_static_all_nodes_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticAllNodesMsg" );
	if (_my_node_map != null) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticAllNodesMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	if (! msg.peer.equals (_fed_node_id)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticAllNodesMsg: " + msg + 
		    " from someone other than the federation.  " +
		    "Dropping it.");
	    return;
	}

	_my_node_map = msg.nodes ();
	_out_times = new HashMap ();
	_timing_map = new HashMap ();

	java.util.Random rand = new java.util.Random(_self_node_id.hashCode());
	for (Iterator i = _my_node_map.keySet ().iterator (); i.hasNext (); )
        {
	    long delay = 1;
            if (! _simulator_running)
            {
                delay = (long) ((_ping_delay * rand.nextDouble()) + 1);
            }

	    NodeId n = (NodeId) i.next ();
	    if (! n.equals (_self_node_id)) {
		StaticPingMsg ping_msg = new StaticPingMsg (n);
		long now = Clock.current_date(_self_node_id).getTime();
		_out_times.put (n, new Long( now + delay ) );
		dispatch (ping_msg, delay);
	    }
	}
    }

    protected void handle_patchwork_probe_msg (PatchworkProbeMsg msg) {
	final String tag = class_tag + ".handle_patchwork_probe_msg";
	if (DEBUG) Debug.printtagln (tag, "processing PatchworkProbeMsg" );

	// if it's bandwidth-related, make a new small one and send it
	if( (msg.type == PatchworkProbeMsg.BANDWIDTH_1) ||
	    (msg.type == PatchworkProbeMsg.BANDWIDTH_2) ) {

	  msg = new PatchworkProbeMsg( msg.peer, PatchworkProbeMsg.MINIMUM_SIZE,
				       msg.type, true );
	} else {
	
	  // otherwise flip the response and send back out
	  msg.response = true;
	  msg.inbound = false;

	}

	dispatch( msg );

    }

    protected void handle_static_ping_msg (StaticPingMsg msg) {
	final String tag = class_tag + ".handle_static_ping_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticPingMsg" );
	if (! _running)
	    dispatch (new StaticPongMsg (msg.peer));
    }

    protected void handle_static_pong_msg (StaticPongMsg msg) {
	final String tag = class_tag + ".handle_static_pong_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticPongMsg" );
	if ((_out_times == null) || (_out_times.isEmpty ())) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticPongMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	long recv_time = System.currentTimeMillis ();

	if (! _out_times.containsKey (msg.peer)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticPongMsg: " + msg + 
		    " from someone unexpected.  " +
		    "Dropping it.");
	    return;
	}

	long send_time = 
	    ((Long) _out_times.remove (msg.peer)).longValue ();
	long delta = recv_time - send_time;

	_timing_map.put (msg.peer, new Long (delta));

	if (DEBUG) Debug.printtagln (tag, 
		"_out_times.size () = " + _out_times.size ());

	if (_out_times.isEmpty ()) {
	    // The timing map is complete.  Fill in our routing tables
	    // and send a ready message.

	    _out_times = null;
	    _routing_tbl = calc_routing_table ();
	    dispatch (new StaticReadyMsg (_fed_node_id));
	}
    }

    protected void handle_static_begin_msg (StaticBeginMsg msg) 
    throws EventHandlerException {
	final String tag = class_tag + ".handle_static_begin_msg";
	if (DEBUG) Debug.printtagln (tag, "processing StaticBeginMsg" );

	if (_running) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got duplicate StaticBeginMsg: " +
		    msg + ".  Dropping it.");
	    return;
	}

	if (! msg.peer.equals (_fed_node_id)) {
	    if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": " + 
		    "got StaticAllNodesMsg: " + msg + 
		    " from someone other than the federation.  " +
		    "Dropping it.");
	    return;
	}

	dispatch (new RoutingTable.Available (_routing_tbl));
	if (DEBUG)
 	  System.out.println(tag+
			   "dispatching routing table: "+
			   _routing_tbl.toString());
	_running = true;
	TapestryReadyMsg ready = new TapestryReadyMsg();
	dispatch(ready);
    }

    public void handleEvent(QueueElementIF item) 
	throws EventHandlerException {
        final String tag = class_tag + ".handleEvent";

	if (DEBUG) Debug.printtagln (tag, "got " + item.getClass () + 
		    " event: " + item + ".");

	if (item instanceof StagesInitializedSignal) {
	    handle_stages_init_signal ();
	}
	else if (item instanceof StaticAllNodesMsg) {
	    handle_static_all_nodes_msg ((StaticAllNodesMsg) item);
	}
	else if (item instanceof PatchworkProbeMsg) {
	    handle_patchwork_probe_msg ((PatchworkProbeMsg) item);
	}
	else if (item instanceof StaticPingMsg) {
	    handle_static_ping_msg ((StaticPingMsg) item);
	}
	else if (item instanceof StaticPongMsg) {
	    handle_static_pong_msg ((StaticPongMsg) item);
	}
	else if (item instanceof StaticBeginMsg) {
	    handle_static_begin_msg ((StaticBeginMsg) item);
	}
    }

    protected RoutingTable calc_routing_table () {
	final String tag = class_tag + ".calc_routing_table";

	RoutingTable result = new RoutingTable (_self_node_id);

	LinkedList [] [] candidates = 
	    new LinkedList [result.digits_per_guid ()] [];
	for (int i = 0; i < result.digits_per_guid (); ++i)
	    candidates [i] = new LinkedList [result.digit_values ()];

	// Sort all the nodes by matching prefix.

	for (Iterator i = _my_node_map.keySet ().iterator (); i.hasNext (); ) {
	    NodeId n = (NodeId) i.next ();
	    if (! n.equals (_self_node_id)) {
		SecureHash g = (SecureHash) _my_node_map.get (n);
		int [] digits = RoutingTable.digits (g);
		int digit = 0;
		for ( ; digit < result.digits_per_guid (); ++digit) 
		    if (digits [digit] != _self_digits [digit]) break;
		Assert.assert (digit != result.digits_per_guid (), 
			tag + ": duplicate GUID.");

		LinkedList l = candidates [digit][digits [digit]];
		if (l == null)
		    l = candidates [digit][digits [digit]] = new LinkedList ();
		l.addLast (n);
	    }
	}

	// Find the closest node for each matching prefix.

	for (int digit = 0; digit < result.digits_per_guid (); ++digit) {
	    for (int value = 0; value < result.digit_values (); ++value) {
		LinkedList l = candidates [digit][value];
		if (_self_digits [digit] == value) {

		    if (DEBUG) Debug.printtagln (tag, "We are our own " +
			    "neighbor for digit " + digit + ", value " +
			    Integer.toHexString (value) + ".");

		    // Add in our own GUID.
		    Assert.assert (l == null, tag + "Shoot!");
		    result.write (digit, value, _self_node_id,
			    (SecureHash) _my_node_map.get (_self_node_id),
				  new Long(0));
		    //now add the rest of the entries
		    if (l != null) {
		      while (!l.isEmpty()) {
			NodeId closest = (NodeId) l.removeFirst();
			Long latency = (Long) _timing_map.get(closest);
			result.addBackup(digit, value, closest, latency,
					 (SecureHash)
					 _my_node_map.get(closest));
		      }
		    } //end if
		}
		else if (l != null) {
		    NodeId closest = (NodeId) l.removeFirst ();
		    long latency = 
			((Long) _timing_map.get (closest)).longValue (); 
		    while (! l.isEmpty ()) {
			NodeId n = (NodeId) l.removeFirst ();
			long nl = ((Long) _timing_map.get (n)).longValue ();
			if (nl < latency) {
			    closest = n;
			    latency = nl;
			}
		    }

		    if (DEBUG) Debug.printtagln (tag, "Our " +
			    "neighbor for digit " + digit + ", value " +
			    Integer.toHexString (value) + " is " +
			    closest + ".");

		    result.write (digit, value, closest,
			    (SecureHash) _my_node_map.get (closest),
				  new Long(latency));

		    result.addDistance(latency, closest,
				       (SecureHash) _my_node_map.get(closest));
		    //now add the rest of the entries
		    while (!l.isEmpty()) {
		      closest = (NodeId) l.removeFirst();
		      result.addBackup(digit, value, closest, 
				       (Long) _timing_map.get(closest),
				       (SecureHash) _my_node_map.get(closest));
		    }
		}
	    }
	}
	return result;
    }

    private NodeId _fed_node_id;
    private SinkIF _sink;
    private boolean _running;
    private SecureHash _self_guid;
    private int [] _self_digits;
    private Map _timing_map;
    private Map _out_times;
    private RoutingTable _routing_tbl;
    private Map _my_node_map;
    private NodeId _self_node_id;
    private int _ping_delay;
    private boolean _simulator_running;
}

