/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;
import java.util.Vector;

/**
 *  @author  Hakim Weatherspoon
 *  @version $Id: QSArray.java,v 1.9 2002/04/04 05:02:48 hweather Exp $
 **/
public class QSArray  extends Object implements Digestible, Cloneable
{
    private QuickSerializable[] _objs;

    public QSArray()
    { 
      _objs = new QuickSerializable[0]; 
    }

    public QSArray(int size)
    { 
      _objs = new QuickSerializable[size]; 
    }

    public QSArray(QuickSerializable obj)
    { 
      _objs = new QuickSerializable[1];
      _objs[0] = obj;//(QuickSerializable)obj.clone();
    }

    public QSArray(QuickSerializable[] objs)
    { 
      _objs = new QuickSerializable[objs.length]; 
      for(int i = 0; i < _objs.length; i++)
        _objs[i] = objs[i]; //(QuickSerializable)objs[i].clone();
    }

    public QSArray (Vector objs) {
      _objs = new QuickSerializable [objs.size ()]; 
      for(int i = 0; i < _objs.length; i++)
        _objs[i] = (QuickSerializable) objs.elementAt (i); 
    }

    public QSArray(byte [] data, int [] offset) throws QSException
    {
      int len = ByteUtils.bytesToInt(data, offset);
      _objs = new QuickSerializable[len];

      for(int i = 0; i < len; i++) {
	  long type_code = ByteUtils.bytesToLong(data, offset);
	  _objs[i] = (QuickSerializable)TypeTable.new_object(type_code, data, offset);
      }
    }

    public int length()
    { 
      return _objs.length; 
    }

    public QuickSerializable[] getQSArray()
    { 
      return _objs; 
    }

    public QuickSerializable get(int index)
    { 
      return _objs[index]; 
    }

    public QuickSerializable set(int index, QuickSerializable obj)
    { 
      return _objs[index] = obj;//(QuickSerializable)obj.clone(); 
    }

    public void append(QuickSerializable obj)
    {
	if(_objs.length == 1 &&
	      (_objs[0] == null || 
	      (_objs[0] instanceof SHA1Hash && ((SHA1Hash)_objs[0]).isNull())))
      {
	_objs[0] = obj;//(QuickSerializable)obj.clone();
      }
      else
      {
	QuickSerializable[] tmpObjs = _objs;
	_objs = new QuickSerializable[tmpObjs.length+1];
	for(int i = 0; i < tmpObjs.length; i++)
	_objs[i] = tmpObjs[i];//(QuickSerializable)tmpObjs[i].clone();
	_objs[tmpObjs.length] = obj;//(QuickSerializable)obj.clone();
      }
    }

    //  methods for ostore.util.QuickSerializable interface
    public void to_bytes(byte [] data, int [] offset)
    {
	ByteUtils.intToBytes(_objs.length, data, offset);
	for(int i = 0; i < _objs.length; i++)
	{
            ByteUtils.longToBytes(TypeTable.type_code(_objs[i]), data, offset);
	    _objs[i].to_bytes(data, offset);
	}
    }

    /** Specified by ostore.util.QuickSerializable */
    public void add_to_digest(SecureHashBuffer buffer)
    {
	buffer.add(_objs.length);
	for(int i = 0; i < _objs.length; i++)
	{
            buffer.add(TypeTable.type_code(_objs[i]));
	    if(!(_objs[i] instanceof Digestible))
		Carp.die("QSArray.add_to_digest: " +
			 "_objs[" + i + "]==" + _objs[i] +
			 " NOT instanceof Digestible.");
	    buffer.add((Digestible)_objs[i]);
	}
    }

    /**
     * Specified by ostore.util.QuickSerializable 
     * @deprecated */
    public int type_code()
    {
	throw new NoSuchMethodError();
	//return ostore.util.Types.QSARRAY; 
    }
    //  end methods for ostore.util.QuickSerializable interface

    //  methods for Object
    public Object clone()
    {
      QuickSerializable[] objs = new QuickSerializable[_objs.length]; 
      for(int i = 0; i < objs.length; i++)
	objs[i] = _objs[i];//(QuickSerializable)_objs[i].clone();
      return objs;
    }

    public boolean equals(Object other)
    {
      if(!(other instanceof QSArray))
	return false;
      if( other == null )
	  return false;
      if( other == this )
	  return true;
      
      QuickSerializable[] otherArray = ((QSArray)other).getQSArray();

      if(_objs.length != otherArray.length)
	return false;

      for(int i = 0; i < _objs.length; i++)
	if(!_objs[i].equals(otherArray[i]))
	  return false;

      return true;
    }

    public int hashCode()
    {
      int result = 0;
      for(int i = 0; i < _objs.length; i++)
	result += _objs[i].hashCode();
	
      return result;
    }

    public String toString()
    {
      String str = new String();
      str += "#(QSArray.toString:  ";
        for(int i = 0; i < _objs.length; i++)
	  str += i + ": " +_objs[i] + " "; 

      str += ")#";

      return str; 
    }
    //  end methods for Object
}
