/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;
import java.util.Date;

/**
 * QSDate.
 *
 * @author     Sean C. Rhea
 * @version    $Id: QSDate.java,v 1.5 2002/03/29 23:26:21 eaton Exp $
 */
public class QSDate implements QuickSerializable, Digestible {

    public QSDate (Date date) {
	this._date = date;
    }

    public QSDate (byte [] buffer, int [] offset) {
	long longdate = ByteUtils.bytesToLong (buffer, offset);
	_date = new Date (longdate);
    }

    public void to_bytes (byte [] buffer, int [] offset) {
	ByteUtils.longToBytes (_date.getTime (), buffer, offset);
    }

    public int type_code () {
	return Types.QSDATE;
    }

    public void add_to_digest(SecureHashBuffer buffer)
    {
        buffer.add(_date.getTime());
        return;
    }

    public Date date () {
	return _date;
    }

    public boolean equals (Object other) {
	if (other == this)
	    return true;
	if (! (other instanceof QSDate))
	    return false;
	return _date.equals (((QSDate) other)._date);
    }

    public String toString () {
	return _date.toString ();
    }

    private Date _date;
}

