/*
 * SimulationMonitorStage.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package soss.core;
import mdw.sandStorm.api.ConfigDataIF;
import mdw.sandStorm.api.EventHandlerException;
import mdw.sandStorm.api.EventHandlerIF;
import mdw.sandStorm.api.QueueElementIF;
import mdw.sandStorm.api.SinkIF;
import mdw.sandStorm.api.SinkException;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;

/**
 * Keeps track of the number of SimulationFinishedEvents that have been
 * dispatched, and exits the simulation when it sees
 * <code>desired_successes</code> such events with exit code 0, or one such
 * event with a non-zero exit code.
 *
 * @author Sean C. Rhea
 * @version $Id: SimulationMonitorStage.java,v 1.1 2001/11/29 20:15:01 srhea Exp $
 */
public class SimulationMonitorStage implements EventHandlerIF {
    
    private static final String class_tag = "SimulationMonitorStage";

    public SimulationMonitorStage (int desired_successes) {
	_desired_successes = desired_successes;
    }

    public void add_classifier (Classifier c) throws ClassNotFoundException {
	final String tag = class_tag + ".add_classifier";
	String [] msg_types = {
	    "soss.core.SimulationFinishedEvent"
	};

	for (int i = 0; i < msg_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (msg_types [i]))) {
		System.err.println (tag + ": could not require type " + 
			msg_types [i]);
		System.exit (1);
	    }
	    c.subscribe (filter, _sink);
	}
    }

    public void init (ConfigDataIF config) throws Exception {
	_sink = config.getStage().getSink();
    }

    public void destroy() throws Exception {
	// NOT USED
    }

    public void handleEvent(QueueElementIF item) 
    throws EventHandlerException {
	final String tag = class_tag + ".handleEvent";

	SimulationFinishedEvent event = (SimulationFinishedEvent) item;
	if (event.exit_code == 0) {
	    _actual_successes += 1;
	    if (_actual_successes == _desired_successes)
		System.exit (0);
	}
	else {
	    System.exit (1);
	}
    }

    public void handleEvents(QueueElementIF items[]) 
	throws EventHandlerException {
	    for(int i=0; i<items.length; i++)
		handleEvent(items[i]);
	}

    private SinkIF _sink;
    private int _actual_successes, _desired_successes;
}

